#!/usr/bin/env python
import unittest
import copy
import json
import os
import manifest
from intervention_test import TestIntervention

import emodpy_tbhiv.interventions.resist_diag as rd
import emod_api.campaign as camp


class Testresistdiag(TestIntervention):
    """
    Test MDRDiagnostic intervention
    """
    def test_default(self):
        super().trigger_treatment_list_test(rd.ResistanceDiagnostic, "MDRDiagnostic", "camp_resistdiag.json", is_multi=False)

    def test_all(self):
        camp.set_schema( self.schema )
        trigger_treatment_list = ['infected']
        sensitivity = 0.1
        specificity = 0.2
        pos_event = 'TBMDRTestPositive'
        neg_event = 'TBMDRTestNegative'
        def_event = 'TBMDRTestDefault'
        treatment_fraction = 0.4
        treatment_fraction_negative_test = 0.8
        start_day = 2
        duration = 10
        property_restrictions_list = ['Demo: 7']
        expected_property_restrictions_list = copy.deepcopy(property_restrictions_list)
        nodeIDs = []
        black_period = 2
        black_trigger = 'Blackout'
        event_name = 'TB Diagnosis MDR'
        event = rd.ResistanceDiagnostic(camp, trigger_treatment_list, sensitivity, specificity, pos_event, neg_event,
                                        def_event, treatment_fraction, treatment_fraction_negative_test, start_day,
                                        duration, property_restrictions_list, nodeIDs, black_period, black_trigger,
                                        event_name)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, "camp_resistdiag.json")
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        ecc = camp_file["Events"][0]["Event_Coordinator_Config"]
        ic = ecc["Intervention_Config"]
        intervention = ic["Actual_IndividualIntervention_Config"]

        self.assertEqual(intervention["Base_Sensitivity"], sensitivity)
        self.assertEqual(intervention["Base_Specificity"], specificity)
        self.assertEqual(intervention["Event_Or_Config"], "Event")
        self.assertEqual(intervention["Negative_Diagnosis_Event"], neg_event)
        self.assertEqual(intervention["Positive_Diagnosis_Event"], pos_event)
        self.assertEqual(intervention["Defaulters_Event"], def_event)
        self.assertEqual(intervention["Treatment_Fraction"], treatment_fraction)
        self.assertEqual(intervention["Treatment_Fraction_Negative_Diagnosis"], treatment_fraction_negative_test)

        self.assertListEqual(ic["Property_Restrictions"], [property_restrictions.replace(" ", "") for
                                                           property_restrictions in
                                                           expected_property_restrictions_list])
        self.assertEqual(ic["Duration"], duration)
        # self.assertEqual(ic["Blackout_Period"], black_period)
        # self.assertEqual(ic["Blackout_Event_Trigger"], black_trigger)
        self.assertEqual(camp_file["Events"][0]["Start_Day"], start_day)
        self.assertEqual(camp.pubsub_signals_pubbing, [pos_event, neg_event, def_event])

    def test_new_intervention_as_file(self, intervention_class_name='MDRDiagnostic'):
        camp_filename = os.path.join(self.output, 'camp_resistdiag_as_file.json')
        super().new_intervention_as_file_test(rd, camp_filename, intervention_class_name, is_multi=False)


if __name__ == '__main__':
    unittest.main()

